from django.db.models.fields import Field
from django.views.generic import (
    CreateView,
    DeleteView,
    DetailView,
    ListView,
    UpdateView,
    TemplateView
)
from django.forms.models import model_to_dict

from dedal.conf import PAGINATE_BY
from dedal.compat import reverse


class DedalBaseMixin(object):
    action_name = None
    model = None
    site = None

    @property
    def model_name(self):
        return self.model._meta.model_name

    def get_template_names(self):
        names = super().get_template_names()
        names.append('dedal/generic_{}.html'.format(self.action_name))
        return names

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['model'] = self.model
        context['models'] = self.site.registered_models
        return context


class ModelListView(DedalBaseMixin, TemplateView):
    template_name = 'dedal/generic_model_list.html'


class DedalListView(DedalBaseMixin, ListView):
    paginate_by = PAGINATE_BY


class DedalReadView(DedalBaseMixin, DetailView):
    def get_context_data(self, *args, **kwargs):
        context = super().get_context_data(*args, **kwargs)
        context['fields'] = model_to_dict(self.object)
        return context


class DedalModelFormMixin(object):
    @property
    def fields(self):
        # TODO: blacklist, editable from field
        return [
            field.name
            for field in self.model._meta.get_fields()
            if isinstance(field, Field)
        ]


class DedalUpdateView(
    DedalModelFormMixin,
    DedalBaseMixin,
    UpdateView
):
    def get_form_class(self):
        return super().get_form_class()

    def get_success_url(self):
        return reverse(
            '{}:read'.format(
                self.model_name
            ), args=(self.object.pk,)
        )


class DedalCreateView(
    DedalModelFormMixin,
    DedalBaseMixin,
    CreateView
):
    def get_success_url(self):
        return reverse(
            '{}:read'.format(
                self.model_name
            ), args=(self.object.pk,)
        )


class DedalDeleteView(
    DedalModelFormMixin,
    DedalBaseMixin,
    DeleteView
):
    action_name = None
    model = None

    def get_success_url(self):
        return reverse('{}:list'.format(self.model_name))



import django

if django.VERSION >= (2, 0):
    from django.urls import clear_url_caches, reverse, NoReverseMatch
else:
    from django.core.urlresolvers import clear_url_caches, reverse, NoReverseMatch



import functools

from dedal import ACTIONS
from dedal.site import site


def register(model=None, actions=ACTIONS):
    if not model:
        return functools.partial(register, actions=actions)
    site.register(model, actions)
    return model

crud = register



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys

version = '1.1.0'

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


def read(filename):
    with open(filename) as f:
        return f.read()

readme = read('README.rst')
history = read('HISTORY.rst').replace('.. :changelog:', '')
requirements = [
    'django>=1.8,<2.2',
    'wheel',
]
test_requirements = read('requirements-test.txt').splitlines()

setup(
    name='django-dedal',
    version=version,
    description="""Fast CRUD builder.""",
    long_description=readme + '\n\n' + history,
    author='Arkadiusz Adamski',
    author_email='arkadiusz.adamski@gmail.com',
    url='https://github.com/ar4s/django-dedal',
    packages=[
        'dedal',
    ],
    include_package_data=True,
    install_requires=requirements,
    tests_require=requirements + test_requirements,
    license="BSD",
    zip_safe=False,
    keywords='django-dedal',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Framework :: Django',
        'Framework :: Django :: 1.6',
        'Framework :: Django :: 1.7',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
)
